""" This class creates and laysout a page of HK parameters and associated plot to be displayed in a tab.


"""

import sys
import re

from PyQt5 import QtWidgets, QtCore, QtGui      #, uic, QtQml, QtQuick
from PyQt5.QtGui import QPainter, QBrush, QPen
from PyQt5.QtCore import Qt

from random import randint
from collections import deque
from pyqtgraph import PlotWidget, plot
import pyqtgraph as pg

#from Euclid import iws_defs, iws, DisplayBox, EuclidHkBox
import iws_defs, iws, DisplayBox, EuclidHkBox

""" These constants refer to the offset of strings in the input feed from the data file """
MIB_NAME = 0
MIB_ID   = 1
HK_UNIT  = 2

fpn = re.compile('[+-]?(\d+(\.\d*)?|\.\d+)([eE][+-]?\d+)?')

maxLen = 50

class GraphPage(QtWidgets.QFrame):

    def __init__(self, parent_frame, json_data):
        """ json_data = {name:name,labels:[{mib_name:mn,mib_id:mi,source:src,unit:v} """
        super(GraphPage, self).__init__()

        #labels = iws.get_json_labels(json_data)
        #self.labels = labels
        self.labels = iws.get_json_labels(json_data)

        self.parameters = {}
        self.raw = False

        # Plot x values
        self.x_data = deque([0.0])

        # Dictionary of y value deques, indexed by mib_name string
        self.y_data = {}

        # pyqtgraph references to each parameter's line plot
        self.y_data_lines = {}
        
        # Plot line colours
        self.pens = [(255,0,0),(0,255,0),(0,0,255),(0,0,0),(255,255,0),(0,255,255),
                     (255,0,255),(255,0,128),(255,128,0),(0,255,128),(128,255,0),(0,128,255)]

        # Dictionary for each parameter's plot colour
        self.y_colours = {}

        #self.debug()

        # y_vals hold the latest engineering value for each parameter
        self.y_vals = {}

        self.maxLen = maxLen
        self.x = 0

        #self.tm_updated = False

        self.setupUi(parent_frame, json_data)


    def debug(self):
        """ Display all the labels """
        for line in self.labels:
            pass
            #print("label: %s %s %s %s" % (line[0], line[1], line[2], line[3]))


    def debug_parameters(self):
        """ Displays all the parameters """
        #print("debug_parameters:")
        for line in self.labels:
            mib = line[1]
            #print(mib, self.parameters[mib][0].text())


    def onClicked(self):
        """ Toggle between showing engineering values and raw values. """
        if self.checkbox.isChecked():
            self.raw = False
            for x in range(0, len(self.parameters)):
                mib_id = self.labels[x][MIB_ID]
                self.parameters[mib_id].value_box.setText(self.parameters[mib_id].eng_string)
        else:
            self.raw = True
            for x in range(0, len(self.parameters)):
                mib_id = self.labels[x][MIB_ID]
                self.parameters[mib_id].value_box.setText(self.parameters[mib_id].raw_string)


    def mibOnClicked(self):
        """ Toggle between displaying parameter MIB names or MIB ids. """
        label_index = 0
        if self.mib_checkbox.isChecked():
            pass
        else:
            label_index = 1
        for x in range(0, len(self.parameters)):
            mib_id = self.labels[x][MIB_ID]
            self.parameters[mib_id].label_box.setText(self.labels[x][label_index])

            
    def update_tm(self, fields):
        """ Update the parameter value box with a new value read from file. """
        try:
            mib_id = fields[1]
            plabel = self.parameters[mib_id].label_box
            vlabel = self.parameters[mib_id].value_box
            unit   = self.parameters[mib_id].unit_string
            if unit == '-':
                unit = ' '
            rawVal = fields[2]
            engVal = fields[3]
            status = fields[4]

            self.parameters[mib_id].raw_string = rawVal
            if self.raw:
                display_string = rawVal
                val = 0.0 # for now
            else:
                if fpn.match(engVal):
                    val = "%3.03f" % float(engVal)
                    display_string = str(val) + " " + unit
                else:
                    val = 0.0 # for now
                    display_string = engVal + " " + unit
            self.y_vals[mib_id] = float(val)
            self.parameters[mib_id].eng_string = display_string

            if (status == "ALARM") or (engVal == "*NO VAL*"):
                newProp = 'ool'
            else:
                newProp = 'ok'

            plabel.setProperty('stat', newProp)
            vlabel.setProperty('stat', newProp)
                
            plabel.setStyle(plabel.style())
            vlabel.setStyle(vlabel.style())

            vlabel.setText(display_string)

            #self.tm_updated = True

        except Exception as e:
            #print(mib_id + str(e))
            pass

    def update_plot(self):
        #if self.tm_updated:

        if self.x < self.maxLen:
            if self.x == 0:
                self.x_data[self.x] = self.x
            else:
                self.x_data.append(self.x)
        else:
            self.x_data.rotate(-1)
            self.x_data[-1] = self.x

        for label in self.labels:
            name = label[MIB_ID]

            if self.x < self.maxLen:
                if self.x == 0:
                    self.y_data[name][self.x] = self.y_vals[name]
                else:
                    self.y_data[name].append(self.y_vals[name])
            else:
                self.y_data[name].rotate(-1)
                self.y_data[name][-1] = self.y_vals[name]

            self.y_data_lines[name].setData(self.x_data, self.y_data[name])

        self.x += 1
        #self.tm_updated = False

# Labels: VIS_CDPU_5V_CPU VDGT2712 rt_slow unit
    def setupUi(self, parent_frame, jdata):
        """ Read in the parameters and layout the boxes. """

        yaxis_label = jdata['yaxis1']['label']
        ymin = jdata['yaxis1']['min']
        ymax = jdata['yaxis1']['max']

        #print(yaxis_label + ' ' + str(ymin) + ' ' + str(ymax))

        #####self.setStyleSheet("background-color:lightpink;")

        label_frame = QtWidgets.QFrame()
        #####label_frame.setStyleSheet("background-color:lightblue;")

        #self.checkbox = QtWidgets.QCheckBox("Engineering", self)
        self.checkbox = QtWidgets.QCheckBox("Engineering", label_frame)
        self.checkbox.setObjectName("checkbox")
        self.checkbox.setChecked(True)
        self.checkbox.setGeometry(5, 5, 100, 16)
        self.checkbox.toggled.connect(self.onClicked)

        #self.mib_checkbox = QtWidgets.QCheckBox("Use MIB name", self)
        self.mib_checkbox = QtWidgets.QCheckBox("Use MIB name", label_frame)
        self.mib_checkbox.setObjectName("mib_checkbox")
        self.mib_checkbox.setChecked(True)
        self.mib_checkbox.setGeometry(110, 5, 115, 16)
        self.mib_checkbox.toggled.connect(self.mibOnClicked)

        mib_names = []
        # Make labels
        column = 0
        row    = 0
        for label_number in range(0, len(self.labels)):
            mib_name = self.labels[label_number][MIB_NAME]
            mib_names.append(mib_name)
            mib_id = self.labels[label_number][MIB_ID]
            (label_box, value_box) = DisplayBox.make_box(label_frame, column, row, mib_name)
            row += 1
            if (label_number + 1) % 21 == 0:
                column += 1
                row = 0

            # Each parameter, identified by its mib id, has a label box, a value box and a unit string
            self.parameters[mib_id] = EuclidHkBox.EuclidHkBox(label_box, value_box, self.labels[label_number][3])

        #self.debug_parameters()

         # Set up plot widget

        plot_frame = QtWidgets.QFrame()
        plot_frame.setStyleSheet("background-color:green")
        self.graphWidget = pg.PlotWidget(plot_frame)
        self.graphWidget.setBackground('w')
        #self.graphWidget.setTitle("Plot Title", color='b', size='150')
        self.graphWidget.setTitle(jdata['name'], color='b', size='150')
        #self.graphWidget.setLabel('left', 'Temperature (C)', color='red', size=30)
        self.graphWidget.setLabel('left', yaxis_label, color='red', size=30)
        self.graphWidget.setLabel('bottom', 'Samples', color='red', size=30)
        #self.graphWidget.addLegend()
        self.graphWidget.showGrid(x=True, y=True)
        self.graphWidget.setYRange(ymin, ymax, padding=0)

        count = 0
        # Set up plot parameters
        for label in self.labels:
            name = label[MIB_ID]
            self.y_data[name] = deque([0.0])
            self.y_vals[name] = 0.0
            color=self.pens[count]
            pen = pg.mkPen(color=color)
            #pen = pg.mkPen(color=self.pens[count])
            #self.y_data_lines[name] = self.graphWidget.plot(self.x_data, self.y_data[name],pen=pen, symbol = '+', symbolSize=8, symbolBrush=(color))
            self.y_data_lines[name] = self.graphWidget.plot(pen=pen, symbol = '+', symbolSize=8, symbolBrush=(color))
            count += 1

        #layout = QtWidgets.QHBoxLayout()
        layout = QtWidgets.QGridLayout()
        layout.addWidget(label_frame, 0, 0, 3, 1)
        #layout.addWidget(plot_frame, 0, 1, 3, 3)
        layout.addWidget(self.graphWidget, 0, 1, 3, 3)
        #layout.setGeometry(QtCore.QRect(10, 10, 100, 100))
        self.setLayout(layout)


    def paintEvent(self, event):
        painter = QPainter()
        painter.begin(self)
        painter.setRenderHint(QPainter.Antialiasing)

        count = 0
        # Set up plot parameters
        for label in self.labels:
            #name = label[MIB_ID]
            #self.y_data[name] = deque([0.0])
            #self.y_vals[name] = 0.0
            color=self.pens[count]
            pen = pg.mkPen(color=color)
            painter.setPen(pen)
            #painter.setPen(QPen(pen, 4, Qt.SolidLine))
            painter.setBrush(Qt.white)
            (x,y) = DisplayBox.label_key_origin(count)
            #print("x = " + str(x) + ", y = " + str(y))
            painter.drawLine(x, y, x + 60, y)
            count += 1
        painter.end()
