--
title: VIS IWS Make Config Program Basic User Manual



author: M. Whillock (m.whillock@ucl.ac.uk). 

date: 24 Feb 2023. 

Copyright © 2020 University College London / MSSL

--

# Introduction


This a mini user guide to the VIS IWS TM HK Make Config (make_config.py) program.

## Prerequisites

* python3
* git

## Installation: Online Deployment

TBD

## Installation: Offline Deployment

The Make Config program is currently bundled in with the TM HK Monitor program.

If this has not alrady been done then choose a location to untar the HKTM_monitor gzipped tar file and untar it.

`tar xvfz HKTM_monitor.tar.gz`


## Make Config

This program is designed to allow the user to create the following for the **HKTM Monitor** program:

- Create complete start-up configurations.
- Define separate hk parameter windows.
- Define separate hk parameter plot windows.
- Define separate hk parameter multi-plot windows.
- Define individual hk parameter tab definitions for the start-up configuration.
- Define individual hk plot tab definitions for the start-up configuration.


This program does not require any conda environment to be set.

### Launching
In an xterm change directory to where the program is installed.
This document assumes that python3 is run using the `python` command.
Use whatever command is required to run python3 on your system.

Launch the program:

`python make_config.py`

Alternatively, run:

`./run_config.sh`

The main window will appear:

![make_config main window](../Pictures/make_config/MainWindow.png)


The main window is made up of several sections:

![make_config main window sections](../Pictures/make_config/MainWindowSections.png)



### Operation

#### General

- The tabs labelled **system** and **packets** in the 'Parameter Tabs' pane allow the user to browse the HK parameters according to the system they refer to or by the HK telemetry packet in which they originate, depending on which tab is selected. These tabs are un-connected with each other: selecting parameters when the **system** tab is selected will not select the corresponding HK parameters in the **packets** tab.
- It does not matter which tab the parameters are chosen from. The parameters have been split like this to give the user options on where to find the parameters.


#### Parameter Selection

- Select a parameter by clicking on it in the '**Parameter Tabs**' area.
- Multiple parameters can be selected by holding down the '**Shift**' key when clicking on the parameters.
	- A click and drag can be used with the '**Shift**' key instead pf selecting parameters one by one.
- Non-contiguous multiple selection is achieved by using the '**Windows**' key (on a pc/linux) or the '**Command**' key (on a Mac).


#### Making a Plot Tab definition
No text yet.

#### Making a Parameter Tab definition
No text yet.

#### Making an HK Monitor configuration
No text yet.


#### Making a Multi-plot Window definition

1. Select the '**Plot**' tab in the '**Tab/Plot Definition Area**'.
2. Choose either the '**Systems**' or '**Packets**' tab from the '**Parameter Tabs**' pane and use the secondary tabs to navigate to the parameters required.
3. Select the required parameters to make a plot. Each parameter will be a curve on the plot. ![Selection](../Pictures/make_config/1_selection.png)
4. Drag the parameters into the box in the '**Tab/Plot Definition Area**'. ![Drag](../Pictures/make_config/2_drop.png)
5. Fill out the name of the plot, the parameter units, the minimum and maximum vales. These are all required for the program to save the definition. The '**Plot Name**' should not contain spaces in it. The minimum and maximum values are required for the HK monitor to set the range of the y-axis; these do not have to be accurate. ![Details](../Pictures/make_config/3_details.png) Once these values have been entered it is possible to save this plot definition by pressing the '**Save**' button in the bottom right hand corner of the '**Tab/Plot Definition Area**'; enter the '**Plot Name**' as the name of the file to keep things consistent. Saving a definition to file is not a requirement for making a multi-plot window definition; the option exists should you want to re-use this particular plot definition in the future for a different multi-plot window definition for example.   
Pressing the '**Clear**' button will remove all the parameters and entered details in case of errors.  

6. To be able to save to a file that can be read by the HK monitor the plot definition must be added to the '**Saved Definitions**' area. This is done by pressing the '**Add**' button in the '**Tab/Plot Definitions Area**'. The plot definition is cleared in the '**Tab/Plot Definition Area**' and the name of the definition is added to the '**Plots**' pane of the '**Saved Definitions**' pane.  ![Add Image](../Pictures/make_config/4_add.png)  

7. Repeat from step 3 for a maximum of 12 times. A previously saved definition can be read by pressing the '**Read**' button in the '**Tab/Plot Definition Area**'.  
The next image shows 4 plot definitions ready to be written to file. This will be displayed in a 2x2 array by the HK monitor; the first two definitions (RPSU\_TEMP & RPSU\_TEMP\_2) will appear in row 1 and the last two will appear in row 2. Clicking on the plot names in the '**Saved Definitions**' pane will display their contents in the '**Tab/Plot Definition Area**'.   ![Save](../Pictures/make_config/5_save.png)  
The number of plots that can be defined for a multi-plot definition is a minimum of 1 and a maximum of 12.  

8. Save the definition to file by:
	- 	selecting the '**Multi Plot Window**' option in the pull down menu item in the '**Control Buttons**' pane.
	- 	entering the name of the definition in the '**Name**' box in the '**Control Buttons**' pane (no spaces in the name). 
	- 	pressing the '**Save**' button in the '**Control Buttons**' pane.  
A save file dialog will appear. To be consistent the file name should be the same as the definition name. Save the file.

	This file will be listed in the open file dialog in the HK Monitor when the '**Open Multi Plot**' button is pressed.  

9. After the definition has been saved the definition name(s) will remain in the '**Plots**' pane of the '**Saved Definitions**' area. They can be deleted by selecting each one in turn and pressing the '**Delete**' button. Don't press the '**Add**' button - just in case.




#### Future Work

- Support for 2 y-axis units in both make_config and the HKTM monitor.
- More support for undo/redo.
