from PyQt5 import QtWidgets, QtGui, QtCore
from PyQt5.QtCore import Qt, QFileInfo
from PyQt5.QtWidgets import QFileDialog, QMessageBox

import sys 

import json
import re

import chooser_defs

XTOP = 0
YTOP = 0
WIDTH = 600
HEIGHT = 600

choices = ['system', 'packets']

system_names = ['CDPU', 'OBS-G', 'PMCU', 'ROE', 'RPSU']
system_tabs = {'CDPU'  : chooser_defs.cdpu_parameters,
               'OBS-G' : chooser_defs.obsg_parameters,
               'PMCU'  : chooser_defs.pmcu_parameters,
               'ROE'   : chooser_defs.roe_parameters,
               'RPSU'  : chooser_defs.rpsu_parameters}

packet_names = ['RT_FAST', 'RT_SLOW', 'RT_SUPERFAST', 'ST_SLOW', 'ST_SUPERFAST']
packet_tabs = {'RT_FAST'      : chooser_defs.rt_fast_parameters,
               'RT_SLOW'      : chooser_defs.rt_slow_parameters,
               'RT_SUPERFAST' : chooser_defs.rt_superfast_parameters,
               'ST_SLOW'      : chooser_defs.st_slow_parameters,
               'ST_SUPERFAST' : chooser_defs.st_superfast_parameters}

class chooserWindow(QtWidgets.QMainWindow):

    def __init__(self):
        super(chooserWindow, self).__init__()
        #print("__init__")
        #self.trees = []

        self.system_trees = {'CDPU'  : None, #[],
                             'OBS-G' : None, #[],
                             'PMCU'  : None, #[],
                             'ROE'   : None, #[],
                             'RPSU'  : None} #[]}

        self.packet_trees = {'RT_FAST'      : None, #[],
                             'RT_SLOW'      : None, #[],
                             'RT_SUPERFAST' : None, #[],
                             'ST_SLOW'      : None, #[],
                             'ST_SUPERFAST' : None} #[]}

        self.parameter_trees = {'system'  : self.system_trees,
                                'packets' : self.packet_trees}

        self.type_choice = 'system'

        self.output_filename = ''
        self.output_dir = './'

        self.initUi()
        

    def initUi(self):
        #print("initUi")

        self.setObjectName("MainWindow")
        self.resize(WIDTH, HEIGHT)

        centralWidget = QtWidgets.QWidget(self)
        centralWidget.setObjectName("centralWidget")
        centralWidget.setAccessibleName("centralWidget")
        #centralWidget.setStyleSheet("background-color:purple;")

        mainMenu = self.menuBar()
        fileMenu = mainMenu.addMenu('File')
        quitAct = QtWidgets.QAction('Quit', self, checkable = False)
        fileMenu.addAction(quitAct)

        self.statusbar = QtWidgets.QStatusBar(self)
        self.statusbar.setObjectName("statusbar")
        self.statusbar.showMessage('Message in statusbar.')
        self.setStatusBar(self.statusbar)

        #self.tabWidget = self.createTabWidget('packet')
        #self.tabWidget = self.createTabWidget('system')
        self.tabWidgetFrame = self.createTabWidgetFrame()

        control_frame = QtWidgets.QFrame()
        control_frame.setObjectName("control_frame")
        #control_frame.setStyleSheet("background-color:yellow;")
        control_frame.setFrameStyle(QtWidgets.QFrame.StyledPanel)

        quitButton = QtWidgets.QPushButton("Quit")
        quitButton.clicked.connect(self.quitClicked)

        exportButton = QtWidgets.QPushButton("Export")
        exportButton.clicked.connect(self.exportClicked)

        cancelButton = QtWidgets.QPushButton("Cancel")
        cancelButton.clicked.connect(self.cancelClicked)

        hbox = QtWidgets.QHBoxLayout()
        #hbox.addStretch(1)
        #hbox.setContentsMargins(0, 0, 0, 0)
        hbox.addWidget(cancelButton)
        hbox.addWidget(exportButton)
        hbox.addStretch(1)
        hbox.addWidget(quitButton)
        control_frame.setLayout(hbox)

        main_layout = QtWidgets.QVBoxLayout()
        main_layout.addWidget(self.tabWidgetFrame, stretch = 1)
        main_layout.addWidget(control_frame, stretch = 0)
        centralWidget.setLayout(main_layout)

        self.setCentralWidget(centralWidget)



    def quitClicked(self):
        quitButtonReply = QtWidgets.QMessageBox.question(self,
                                                         'PyQt5 message', 'Are you sure you want to quit the program?',
                                                         QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No, QtWidgets.QMessageBox.No)
        if quitButtonReply == QtWidgets.QMessageBox.Yes:
            self.close()
            exit(1)


    def clearAllClicked(self, b):
        current_tab = self.masterTabWidget.currentIndex()
        type = 'system' if current_tab == 0 else 'packets'
        names = system_names if type == 'system' else packet_names
        for name in names:
            for item in self.parameter_trees[type][name].findItems("", Qt.MatchContains | Qt.MatchRecursive):
                item.setCheckState(0, Qt.Unchecked)


    # https://doc.qt.io/qtforpython/PySide2/QtCore for findItems flags
    def deselectAllClicked(self, b):
        tab_name = b.objectName()
        current_tab = self.masterTabWidget.currentIndex()
        type = 'system' if current_tab == 0 else 'packets'
        for item in self.parameter_trees[type][tab_name].findItems("", Qt.MatchContains | Qt.MatchRecursive):
            #if item.checkState(0) != Qt.Unchecked:
            item.setCheckState(0, Qt.Unchecked)
                #print(item.text(0) + " " + str(item.childCount()))
                # Feature in checkbox - doesnt set to unchecked if partially set. This doesn't clear it.


    def selectAllClicked(self, b):
        tab_name = b.objectName()
        current_tab = self.masterTabWidget.currentIndex()
        type = 'system' if current_tab == 0 else 'packets'
        #the_tree = self.parameter_trees[type][tab_name][0]
        #the_tree = self.parameter_trees[type][tab_name]
        for item in self.parameter_trees[type][tab_name].findItems("", Qt.MatchContains | Qt.MatchRecursive):
            item.setCheckState(0, Qt.Checked)


    def saveFileDialog(self):
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        self.output_filename, _ = QFileDialog.getSaveFileName(self, "QFileDialog.getSaveFileName()", self.output_dir, "csv files (*.csv)", options = options)
        if self.output_filename:
            print(f"File {self.output_filename} selected")
            suffix = QFileInfo(self.output_filename).suffix()
            if suffix != "csv":
                msg = QtWidgets.QMessageBox()
                msg.setIcon(QtWidgets.QMessageBox.Information)
                msg.setText("The file needs to end with .csv")
                msg.setDetailedText("The file will not be created or written to.")
                msg.setWindowTitle("Error")
                msg.setStandardButtons(QtWidgets.QMessageBox.Ok)
                retval = msg.exec_()
                self.output_filename = ''
            else:
                print(f"Using {self.output_filename}")
                self.output_dir = QFileInfo(self.output_filename).absoluteFilePath()
                pass
        else:
            print(f"No file selected")
            self.output_filename = ''

    # Rename to exportClicked
    def exportClicked(self):
        current_tab = self.masterTabWidget.currentIndex()
        type = 'system' if current_tab == 0 else 'packets'
        names = system_names if type == 'system' else packet_names
        items_checked = 0
        items = []
        for name in names:
            for item in self.parameter_trees[type][name].findItems("", Qt.MatchContains | Qt.MatchRecursive):
                if (item.checkState(0) > 0):
                    if item.childCount() == 0:
                        items_checked += 1
                        items.append(item)
        if items_checked != 0:
            print(items)
            self.saveFileDialog()
            if self.output_filename == '':
                return
            f = open(self.output_filename, "w")
            for item in items:
                print (item.text(0),item.text(1), item.text(2), item.checkState(0))
                f.write(item.text(0) + ", " + item.text(1) + ", " + item.text(2) + "\n")
            f.close()
            #for name in names:
                #for item in self.parameter_trees[type][name].findItems("", Qt.MatchContains | Qt.MatchRecursive):
                    #if (item.checkState(0) > 0):
                        #if item.childCount() == 0:
                            #print (item.text(0),item.text(1), item.text(2), item.checkState(0))
        else:
            print("No items checked")
            msg = QtWidgets.QMessageBox()
            msg.setIcon(QtWidgets.QMessageBox.Information)
            msg.setText("No parameters are selected!")
            msg.setDetailedText("The file will not be created or written to.")
            msg.setWindowTitle("Error")
            msg.setStandardButtons(QtWidgets.QMessageBox.Ok)
            retval = msg.exec_()


    def cancelClicked(self):
        pass


    def createTab(self, type):
        #font = QtGui.QFont()
        #font.setPointSize(11)
        #font.setBold(True)
        #font.setWeight(75)

        names = system_names if type == 'system' else packet_names
        tabs =  system_tabs  if type == 'system' else packet_tabs
        tabWidget = QtWidgets.QTabWidget()
        #tabWidget.setFont(font)
        tabWidget.setObjectName("tabWidget")
        for name in names:
            tab = QtWidgets.QWidget(tabWidget)
            tab.setObjectName(name + '_tab')
            #self.parameter_trees[type][name].append(self.createTree(tab, name, tabs[name], type))
            self.parameter_trees[type][name] = self.createTree(tab, name, tabs[name], type)
            tabWidget.addTab(tab, name)
        
        tabWidget.setCurrentIndex(0)
        return tabWidget


    def createTabWidgetFrame(self):
        frame = QtWidgets.QFrame()
        #frame.setStyleSheet("background-color:yellow;")
        frame.setFrameStyle(QtWidgets.QFrame.StyledPanel)
        #font = QtGui.QFont()
        #font.setPointSize(11)
        #font.setBold(True)
        #font.setWeight(75)

        self.masterTabWidget = QtWidgets.QTabWidget()
        #self.masterTabWidget.setStyleSheet("background-color:lightgreen;")
        #self.masterTabWidget.setFont(font)
        for type in choices:
            self.masterTabWidget.addTab(self.createTab(type), type)

        layout = QtWidgets.QVBoxLayout()
        layout.addWidget(self.masterTabWidget)

        clearButton = QtWidgets.QPushButton("Clear All")
        clearButton.clicked.connect(lambda:self.clearAllClicked(clearButton))
        clearButtonFrame = QtWidgets.QFrame()
        clearButtonLayout = QtWidgets.QHBoxLayout()
        clearButtonLayout.addWidget(clearButton)
        clearButtonLayout.addStretch(1)
        clearButtonFrame.setLayout(clearButtonLayout)
        layout.addWidget(clearButtonFrame)

        frame.setLayout(layout)
        return frame


    def createTree(self, tab, label, parameter_file, type):
        layout = QtWidgets.QVBoxLayout()
        tree = QtWidgets.QTreeWidget()
        tree.setColumnCount(3)
        tree.setHeaderLabels([label + " Parameters", "MIB ID", "Source"])
        # Yuck
        tree.setColumnWidth(0, 250)

        layout.addWidget(tree, stretch = 0)

        bottom_button_frame = QtWidgets.QFrame()
        deSelectAllButton = QtWidgets.QPushButton('Deselect All')
        deSelectAllButton.setObjectName(label)
        deSelectAllButton.clicked.connect(lambda:self.deselectAllClicked(deSelectAllButton))
        selectAllButton = QtWidgets.QPushButton('Select All')
        selectAllButton.setObjectName(label)
        selectAllButton.clicked.connect(lambda:self.selectAllClicked(selectAllButton))
        bottom_frame_layout = QtWidgets.QHBoxLayout()
        bottom_frame_layout.addWidget(deSelectAllButton)
        bottom_frame_layout.addWidget(selectAllButton)
        bottom_frame_layout.addStretch(1)
        bottom_button_frame.setLayout(bottom_frame_layout)
        layout.addWidget(bottom_button_frame, stretch = 0)

        tab.setLayout(layout)
        
        seen_parent = False
        blank = re.compile('^$')
        comment = re.compile('^\?')
        alt_comment = re.compile('^#')
        minus = re.compile('^---')
        pluses = re.compile('^\+\+\+')
        
        try:
            with open(parameter_file, 'r') as f:
                for line in f:
                    # txt
                    #items = line.split()
                    # csv
                    items = line.split(',')
                    if blank.match(line):
                        seen_parent = False
                        continue
                    if comment.match(line):
                        continue
                    if alt_comment.match(line):
                        continue
                    if minus.match(line):
                        seen_parent = True
                        parent = QtWidgets.QTreeWidgetItem(tree) 
                        parent.setText(0, items[1]) 
                        parent.setText(1, items[2])
                        parent.setText(2, items[3])
                        parent.setFlags(parent.flags() | Qt.ItemIsTristate | Qt.ItemIsUserCheckable) 
                        continue
                    if pluses.match(line):
                        seen_parent = True
                        parent = QtWidgets.QTreeWidgetItem(tree) 
                        parent.setText(0, items[1]) 
                        parent.setText(1, items[2])
                        parent.setText(2, items[3])
                        parent.setFlags(parent.flags() | Qt.ItemIsTristate | Qt.ItemIsUserCheckable) 
                        continue
                    if seen_parent:
                        child = QtWidgets.QTreeWidgetItem(parent) 
                        child.setFlags(child.flags() | Qt.ItemIsUserCheckable) 
                        child.setText(0, items[0]) 
                        child.setText(1, items[1])
                        child.setText(2, items[2])
                        child.setCheckState(0, Qt.Unchecked) 
                    else:
                        parent = QtWidgets.QTreeWidgetItem(tree) 
                        parent.setText(0, items[0]) 
                        parent.setText(1, items[1])
                        parent.setText(2, items[2])
                        parent.setFlags(parent.flags() | Qt.ItemIsUserCheckable) 
                        parent.setCheckState(0, Qt.Unchecked) 
                        seen_parent = False
                        
        except Exception as e:
            print("EXCEPTION")
        finally:
            f.close()

        return tree
        
        
if __name__ == '__main__': 

    app = QtWidgets.QApplication([])
    appWindow = chooserWindow()
    appWindow.show()
    sys.exit(app.exec_()) 
